%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VELOCITY.M

% This is a MATLAB program, written by Bruce E. Hansen.
% It replicates Table 4 and Figures 1and 2 from my paper
% "The Grid Bootstrap and the Autoregressive Model".

% If you have questions, please contact me at

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
function velocity()
global determ;
global n;
g=200; 	% 200 %
b=3000;    % 1999 %
bq=3000;   % 1999 %
c=.9;           % confidence level %
width=6;
if c==.9 aa=1.645; end;

cc=[((1-c)/2);((c+1)/2)];
rc=cc((length(cc(:,1)):-1:1),:);
load stock_t.mat;

load nplosser.txt;
y=nplosser(1215:1334);
n=length(y(:,1));

xs=1869;
x=xs:xs+n-1;
plot(x,y);
grid on;
title('Velocity: 1869-1988');
saveas(gcf,'Vel.eps','psc2');

out=fopen('velocity.txt','wt');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(out,'Nelson-Plosser Velocity\nSample:   1869-1988\n');
fprintf(out,'Gridpoints:   %f\n',g);
fprintf(out,'Standard Bootstraps:    %u\n',b);
fprintf(out,'Grid Bootstraps:    %f\n',bq);
fprintf(out,'Confidence Level:   %f\n\n',c);

determ=[ones(n-1,1),(1:n-1)'];
d0=[[1,0];determ];
yt=y-d0*(y'/d0')';
y0=yt(1);
xname=['Asymptotic';'_____Stock';'_____Efron';'___Perct-t';'____Kilian';'____Grid-a';'____Grid-t'];
nt=length(xname(:,1));

[theta,se,e]=b_est(y);
ahat=theta(3);
sehat=se(3);
kkname=['Intercpt';'___Trend';'___Y(-1)'];
fprintf(out,'Variable     Estimate     S.E\n');
for i=1:3
    fprintf(out,'%s     %f   %f\n',kkname(i,:),theta(i),se(i));
end;
clear kkname;
ends=zeros(nt,2);

% Asymptotic methods %
ui=ahat+[-1;1]*sehat*aa;
ends(1,:)=ui';
ui=unitci(((ahat-1)'/sehat')',c,stock_t);  % Stock %
ends(2,:)=(1+ui/(n-1))';

% Standard Bootstraps %
boota=zeros(b,1);
boott=zeros(b,1);
j=1;
while j<=b
    [thetab,seb,eb]=b_est(ar1(y0,ahat,e));
    boota(j)=thetab(3);
    boott(j)=((thetab(3)-ahat)'/seb(3)')';
    j=j+1;
end;

% efron %
ef=quantile(boota,cc);
ends(3,:)=ef;
% percentile t %
qt=quantile(boott,rc);
pt=ahat-qt*sehat;
ends(4,:)=pt;

% Kilian bootstrap %
if ahat>=1
    bc=ahat;
    bias=0;
else
    bias=mean(boota)'-ahat;
    bc=ahat-bias;
    if bc>=1
        bc=1;
        bias=ahat-bc;
    end;
end;

j=1;
while j<=b
    [thetab,seb,eb]=b_est(ar1(y0,bc,e));
    boota(j)=thetab(3)-bias;
    j=j+1;
end;
ends(5,:)=quantile(boota,cc);

% Grid Bootstrap %
bootq=zeros(bq,g);
bootse=zeros(bq,g);
a0=(.84:.22/g:.22*(g-1)/g+.84)';
ak=1;
while ak<=g
    alphak=a0(ak);
    j=1;
    while j<=bq
        [thetab,seb,eb]=b_est(ar1(y0,alphak,e));
        bootq(j,ak)=thetab(3);
        bootse(j,ak)=seb(3);
        j=j+1;
    end;
    ak=ak+1;
end;
ba=bootq-ones(bq,1)*a0';
ts=ba./bootse;
grid_p=kq_reg(ba,a0,cc);
grid_t=kq_reg(ts,a0,cc);
ah0=ahat-a0;
ts=(ah0'/sehat')';
ends(6,:)=g_ends(a0,ah0,grid_p);
ends(6,:)
ends(7,:)=g_ends(a0,ts,grid_t);
fprintf(out,'\n Confidence Intervals for AR(1) Parameter\n');
fprintf(out,'Method     Left End     Right End\n');
for i=1:length(xname(:,1))
    fprintf(out,'%s   %f   %f\n',xname(i,:),ends(i,1),ends(i,2));
end;
fprintf(out,'------------------------------------------\n\n');
fclose(out);

% Plot 1 - Efron Construction %
z=zeros(g,1);
eu=ef(2)-ahat;el=ef(1)-ahat;
l=ends(6,1);u=ends(6,2);
ql=ahat-l;qu=ahat-u;
hpu=ahat+eu;hpl=ahat+el;
figure;
title('Velocity: 1869-1988');
xlabel('90% Percentile Confidence Interval');
ylabel('Bootstrap Quantile');
xlim([.84,1]);
ylim([-.15,.05]);
hold on;
% Points %
plot(ahat,0,'o');
plot(ahat,eu,'bd');
plot(ahat,el,'bd');
plot(hpu,eu,'rs');
plot(hpl,el,'rs');
% Lines %
plot([hpl;ahat],[el;el],':');
plot([ahat;ahat],[el;eu],':');
plot([ahat;hpu],[eu;eu],':');
% Arrows %
plot([hpu;hpu],[eu;0],':');plot(hpu,0,'*');
plot([hpl;hpl],[el;0],':');plot(hpl,0,'*');
plot(a0,z,a0,a0-ahat,a0,grid_p(:,2),a0,grid_p(:,1));
hold off;
saveas(gcf,'vel1.eps','psc2');

% Plot 2 - Grid-alpha Construction (Figure 2) %
figure;
z=zeros(g,1);
eu=ef(2)-ahat;el=ef(1)-ahat;
l=ends(6,1);u=ends(6,2);
ql=ahat-l;qu=ahat-u;
title('Figure.2  Velocity: 1869-1988');
xlabel('90% Grid-alpha Confidence Interval');
ylabel('Bootstrap Quantile');
hold on;
xlim([.9 1.06]);
ylim([-.2 .05]);
% Points %
plot(ahat,0,'o');
plot(l,ql,'+');
plot(u,qu,'+');
% Arrows %

plot([l,l],[ql,0]);plot(l,0,'*');
plot([u,u],[qu,0]);plot(u,0,'*');
plot(a0,ah0,a0,z,a0,grid_p(:,2),a0,grid_p(:,1));
hold off;
saveas(gcf,'vel2.eps','psc2');

% Plot 3 - Percentile-t and Grid-t Construction (Figure 1) %
figure;
hold on;
ylabel('Bootstrap Quantile');
l=ends(7,1);u=ends(7,2);
eu=qt(1);el=qt(2);
ql=((ahat-l)'/sehat')';qu=((ahat-u)'/sehat')';
hl=ahat-sehat*el;
hu=ahat-sehat*eu;
% Points %
plot(ahat,0,'o');
plot(ahat,eu,'bd');
plot(ahat,el,'bd');
plot(hl,el,'rs');
plot(hu,eu,'rs');
plot(l,ql,'rs');
plot(u,qu,'rs');
% Lines %
plot([hl,ahat],[el,el],':');
plot([ahat,ahat],[el,eu],':');
plot([ahat,hu],[eu,eu],':');
% Arrow %
plot([l,l],[ql,0]);plot(l,0,'*');
plot([u,u],[qu,0]);plot(u,0,'*');
plot([hu,hu],[eu,0],':');plot(hu,0,'*');
plot([hl,hl],[el,0],':');plot(hl,0,'*');
xlim([.9 1.06]);
ylim([-4 2]);
xlabel('90% Percentile-t and Grid-t Confidence Intervals');
title('Figure.1 Velocity: 1869-1988');
plot(a0,ts,a0,z,a0,grid_t(:,2),a0,grid_t(:,1));
hold off;
saveas(gcf,'vel3.eps','psc2');



% Procedures %
function [theta,se,e]=b_est(yy);
global determ;
global n;
x=[determ,yy(1:n-1)];
y=yy(2:n);
xx=inv(x'*x);
theta=xx*(x'*y);
e=y-x*theta;
se=sqrt(diag(xx*(e'*e)/(n-4)));


function y=ar1(y0,a,e);
global n;
nn=length(e(:,1));
if abs(a)<1
    y00=y0;
else
    y00=0;
end;
et=e(ceil(nn*unifrnd(0,1,n,1)));
y=y00;
for iii=(length(y00(:,1))+1):(length(et(:,1)))
     datbb=et(iii,:);
     for jj=1:length(y00(:,1))
          datbb=datbb+a(jj,:).*y(iii-jj,:);
     end;
     y=[y;datbb];
end;


function [g1,g2]=cross(a,x,y);
ms=NaN;      
rn=length(a(:,1));
r=(x<y);
d=[0;(r(2:rn)-r(1:rn-1))];
s=(1:rn)';
ind=0;
id=(d==1);
for i=1:length(id)
    if id(i)==1
        if ind==0
            li=s(i);
            ind=1;
        else
            li=[li;s(i)];
        end;
    end;
end;
if 1-(isnan(sum(li)))
    al=a(li-1);
    au=a(li);
    bl=x(li-1)-y(li-1);
    bu=y(li)-x(li);
    g1=(bu.*al+bl.*au)./(bl+bu);
else
    g1=ms;
end;

ind=0;
id=(d==-1);
for i=1:length(id)
    if id(i)==1
        if ind==0
            li=s(i);
            ind=1;
        else
            li=[li;s(i)];
        end;
    end;
end;
if max(id==0) li=NaN; end;
if 1-(isnan(sum(li)))
    al=a(li-1);
    au=a(li);
    bl=y(li-1)-x(li-1);
    bu=x(li)-y(li);
    g2=(bu.*al+bl.*au)./(bl+bu);
else
    g2=ms;
end;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC G_ENDS
% Format: 	
% 	[l,u]=g_ends(a0,ta,grid);
% Inputs:  
% 	a0	rx1 list of gridpoints
%	ta	rx1 test statistic line
%	grid	rx2 lower and upper quantiles
% Output:
% 	l	lower endpoint of confidence interval
% 	u	upper endpoint of confidence interval
function [lu]=g_ends(a0,ta,grid);
[l1,l2]=cross(a0,ta,grid(:,1));
[u1,u2]=cross(a0,ta,grid(:,2));
is=sortrows([l1;l2;u1;u2],1);
if length(is(1,:))>1
   iis=isnan(sum(is')');
else
   iis=isnan(is);
end;
ind=0;
for i=1:length(iis)
    if iis(i)==0;
        if ind==0
            s=is(i);
            ind=1;
        else
            s=[s;is(i)];
        end;
    end;
end;
l=min(s);
u=max(s);
if l==min(u2) l=min(a0);end;
if l==min(l1) l=min(a0);end;
if u==max(l2) u=max(a0);end;
if u==max(u1) u=max(a0);end;
lu=[l,u];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC QUANTILE
% Format: 	
% 	q=quantile(x,c);
% Inputs:  
% 	x	nxr data matrix
% 	c	px1 list of quantiles
% Output:
% 	q	rxp matrix of quantiles

function q=quantile(x,c);
[b,r]=size(x);
xx=x;
j=1;
while j<=r
    xx(:,j)=sortrows(x(:,j),1);
    j=j+1;
end;
bc=b*c;
i=floor(bc);
i=i+(i==0);
q=xx(i,:).*((i==bc)*ones(1,length(xx(1,:))))+((xx(i,:)+xx(i+1,:))/2).*((1-(i==bc))*ones(1,length(xx(1,:))));
q=q';

function [c1,c2,c3]=c_out(cc,alpha);
c1=(cc(1)>alpha);
c2=(cc(2)<alpha);
c3=(cc(2)-cc(1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC EPI
% Format: 	
% 	w=epi(a,h,leave);
% Inputs:  
% 	a	nx1 vector of points to evaluate kernel
% 	h	scalar bandwidth
% 	leave	set to 1 to impose leave-one-out kernel estimation
% Output:
% 	w	nxn matrix of kernel weights

function w=epi(a,h,leave);
aa=a*ones(1,length(a))-ones(length(a),1)*a';
u=(aa'/h)';
w=(1-u.^2).*(abs(u)<=1);

if leave==1
    for i=1:length(w(:,1))
        w(i,i)=0;
    end;
end;
w=w./(sum(w')'*ones(1,length(w(1,:))));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC KQ_REG
% Format: 	
% 	qh=kq_reg(y,x,c);
% Inputs:  
% 	y	nxr data matrix
% 	x	rx1 vector of points to evaluate kernel regression
% 	c	px1 list of quantiles
% Output:
% 	qh	rxp matrix of kernel-smoothed quantiles

function qh=kq_reg(y,x,c);
hn=5;
cn=length(c(:,1));
hh=(1.5:hn+0.5)'*(x(2)-x(1));
check=zeros(hn,cn);
q=quantile(y,c);
hi=1;
while hi<=hn
    h=hh(hi);
    if x(2)-x(1)<h
        qh=epi(x,h,1)*q;
        check(hi,:)=sum((q-qh).^2);
    else
        check(hi,:)=sum(q.^2);
    end;
    hi=hi+1;
end;
[temp,hi]=min(check);
h=hh(hi);
clear temp;
qh=zeros(length(x(:,1)),cn);
ci=1;
while ci<=cn
    qh(:,ci)=(epi(x,h(ci),0)*q(:,ci));
    ci=ci+1;
end;

% Stock's proc for computing local-to unity confidence intervals %
function c10=unitci(tdf,clevel,cv);
alpha0=0.5*(1-clevel);
sel0=(fix(1000*cv(1,:)'+.01)==fix(1000*alpha0+.01));
sel1=(fix(1000*cv(1,:)'+.01)==fix(1000*(1-alpha0)+.01));
i0=sum((1:length(cv(1,:)))'.*sel0)';
i1=sum((1:length(cv(1,:)))'.*sel1)';
cc=cv(2:length(cv(:,1)),1);
cv0=cv(2:length(cv(:,1)),i0);
cv1=cv(2:length(cv(:,1)),i1);
d1=(cv0<tdf);
d2=(cv1>tdf);
dd=d1.*d2;
tag=0;
if i0==i1 % return median -- 0-width CI %
    if (sum(d1)>0) & (sumc(d2)>0)
        ind=0;
        for iii=1:length(cc(:,1))
            if d1(iii)==1
                if ind==0
                    jj=iii;
                    ind=1;
                else
                    jj=[jj;iii];
                end;
            end;
        end;
        if length(jj(:,1))==1
            j0=jj;
        else
            j0=max(jj)';
        end;
        j1=j0+1;
    elseif sum(d1)==0
        c0=-99.99;
        c1=c0;
        tag=1;
    elseif sum(d2)==0 & tag==0
        c0=99.99;
        c1=c0;
        tag=1;
    end;
elseif sum(dd)>0 % nonempty CI %
        ind=0;
        for iii=1:length(cc(:,1))
            if d1(iii)==1
                if ind==0
                    jj=iii;
                    ind=1;
                else
                    jj=[jj;iii];
                end;
            end;
        end;
        if length(jj(:,1))==1
            j0=jj;
        else
            j0=max(jj)';
        end;
        if length(jj(:,1))==1
            j1=jj;
        else
            j1=min(jj)';
        end;
    
else % empty confidence interval -- outside the range covered by cv %
    if sum(d1)==0;
        c0=-99.99;
        c1=-99.99;
    elseif sum(d2)==0
        c0=99.99;
        c1=99.99;
    end;
    tag=1;
end;

if tag==0
if j0<length(cc(:,1))
    wt0=(tdf-cv0(j0,1))/(cv0(j0+1,1)-cv0(j0,1));
    c0=(1-wt0)*cc(j0,1)+wt0*cc(j0+1,1);
else
    wt0=0;
    c0=99.99;
end;

if j1>1
    wt1=(tdf-cv1(j1-1,1))/(cv1(j1,1)-cv1(j1-1,1));
    c1=(1-wt1)*cc(j1-1,1)+wt1*cc(j1,1);
else
    wt1=0;
    c1=-99.99;
end;
end;
c10=[c1;c0];

